@@ -752,3 +752,162 @@ transplantEnv = function(
   }
   invisible(out)
 }
+
+
+#' Add amplitude modulation
+#'
+#' Adds sinusidal or logistic amplitude modulation to a sound. This produces
+#' additional harmonics in the spectrum at ±am_freq around each original
+#' harmonic and makes the sound rough. The optimal frequency for creating a
+#' perception of roughness is ~70 Hz (Fastl & Zwicker "Psychoacoustics").
+#' Sinusoidal AM creates a single pair of new harmonics, while non-sinusoidal AM
+#' creates more extra harmonics (see examples).
+#' @param x path to a .wav file or a vector of amplitudes with specified
+#'   samplingRate
+#' @inheritParams soundgen
+#' @param checkFormat only FALSE when called internally by soundgen()
+#' @param plot if TRUE, plots the amplitude modulation
+#' @export
+#' @examples
+#' sound1 = soundgen(pitch = c(200, 300), addSilence = 0)
+#' s1 = addAM(sound1, 16000, amDep = c(0, 50, 0), amFreq = 75, plot = TRUE)
+#' # playme(s1)
+#' \dontrun{
+#' # Parameters can be specified as in the soundgen() function, eg:
+#' s2 = addAM(sound1, 16000,
+#'          amDep = list(time = c(0, 50, 52, 200, 201, 300),
+#'                       value = c(0, 0, 35, 25, 0, 0)),
+#'          plot = TRUE, play = TRUE)
+#'
+#' # Sinusoidal AM produces exactly 2 extra harmonics at ±am_freq around each f0
+#' harmonic:
+#' s3 = addAM(sound2, 16000, amDep = 30, amFreq = c(50, 80), amType = 'sine',
+#'            plot = TRUE, play = TRUE)
+#' spectrogram(s3, 16000, windowLength = 150, ylim = c(0, 2))
+#'
+#' # Non-sinusoidal AM produces multiple new harmonics, which can resemble
+#' subharmonics...
+#' s4 = addAM(sound2, 16000, amDep = 70, amFreq = 50, amShape = -1,
+#'            plot = TRUE, play = TRUE)
+#' spectrogram(s4, 16000, windowLength = 150, ylim = c(0, 2))
+#'
+#' # ...but more often look like sidebands
+#' sound3 = soundgen(sylLen = 600, pitch = c(800, 1300, 1100), addSilence = 0)
+#' s5 = addAM(sound3, 16000, amDep = c(0, 30, 100, 40, 0),
+#'            amFreq = 105, amShape = -.3,
+#'            plot = TRUE, play = TRUE)
+#' spectrogram(s5, 16000, ylim = c(0, 5))
+#'
+#' # Feel free to add AM stochastically:
+#' s6 = addAM(sound2, 16000,
+#'            amDep = rnorm(10, 40, 20), amFreq = rnorm(20, 70, 20),
+#'            plot = TRUE, play = TRUE)
+#' spectrogram(s6, 16000, windowLength = 150, ylim = c(0, 2))
+#'
+#' # If am_freq is locked to an integer ratio of f0, we can get subharmonics
+#' # For ex., here is with pitch 400-600-400 Hz (soundgen interpolates pitch
+#' # on a log scale and am_freq on a linear scale, so we align them by extracting
+#' # a long contour on a log scale for both)
+#' con = getSmoothContour(anchors = c(400, 600, 400),
+#'                        len = 20, thisIsPitch = TRUE)
+#' s = soundgen(sylLen = 1500, pitch = con, amFreq = con/3, amDep = 30,
+#'              plot = TRUE, play = TRUE, ylim = c(0, 3))
+#' }
+addAM = function(x,
+                 samplingRate = NULL,
+                 amDep = 25,
+                 amFreq = 30,
+                 amType = c('logistic', 'sine')[1],
+                 amShape = 0,
+                 invalidArgAction = c('adjust', 'abort', 'ignore')[1],
+                 plot = FALSE,
+                 play = FALSE,
+                 checkFormat = TRUE) {
+  # import audio
+  if (class(x)[1] == 'character') {
+    extension = substr(x, nchar(x) - 2, nchar(x))
+    if (extension == 'wav' | extension == 'WAV') {
+      sound_wav = tuneR::readWave(x)
+    } else if (extension == 'mp3' | extension == 'MP3') {
+      sound_wav = tuneR::readMP3(x)
+    } else {
+      stop('Input not recognized: must be a numeric vector or wav/mp3 file')
+    }
+    samplingRate = sound_wav@samp.rate
+    sound = as.numeric(sound_wav@left)
+  } else if (class(x)[1] == 'numeric') {
+    if (is.null(samplingRate)) {
+      stop ('Please specify samplingRate, eg 44100')
+    } else {
+      sound = x
+    }
+  }
+  len = length(sound)
+
+  # check the format
+  if (checkFormat) {
+    anchors = c('amDep', 'amFreq', 'amShape')
+    for (anchor in anchors) {
+      assign(anchor, reformatAnchors(get(anchor)))
+    }
+  }
+
+  # vectorize
+  amPar_vect = c('amDep', 'amFreq', 'amShape')
+  # just to get rid of of NOTE on CRAN:
+  amDep_vector = amFreq_vector = amShape_vector = vector()
+  for (p in amPar_vect) {
+    p_unique_value = unique(get(p)$value)
+    if (length(p_unique_value) > 1) {
+      if (invalidArgAction == 'ignore') {
+        valueFloor_p = valueCeiling_p = NULL
+      } else {
+        valueFloor_p = permittedValues[p, 'low']
+        valueCeiling_p = permittedValues[p, 'high']
+      }
+      p_vectorized = getSmoothContour(
+        anchors = get(p),
+        len = len,
+        interpol = 'approx',
+        valueFloor = valueFloor_p,
+        valueCeiling = valueCeiling_p
+      )
+      # plot(p_vectorized, type = 'l')
+      assign(paste0(p, '_vector'), p_vectorized)
+    } else {
+      assign(paste0(p, '_vector'), p_unique_value)
+    }
+  }
+
+  # prepare am vector
+  if (amType == 'sine') {
+    if (length(amFreq_vector) == 1) {
+      int = amFreq_vector * (1:len)
+    } else {
+      int = cumsum(amFreq_vector)
+    }
+    sig = .5 + .5 * cos(2 * pi * int / samplingRate)
+  } else {
+    sig = getSigmoid(len = len,
+                     samplingRate = samplingRate,
+                     freq = amFreq_vector,
+                     shape = amShape_vector)
+  }
+  # plot(sig, type = 'l')
+  # sig is on a scale [0, 1]
+  am = 1 - sig * amDep_vector / 100
+  sound_am = sound * am
+
+  if (plot) {
+    osc(x = am,
+        samplingRate = samplingRate,
+        main = 'Amplitude modulation',
+        xlab = 'Time, ms',
+        ylab = '',
+        ylim = c(0, 1),
+        midline = FALSE)
+  }
+  if (play) playme(sound_am, samplingRate)
+  invisible(sound_am)
+}
+
